@@ -54,6 +54,7 @@
 #include <exchange-hierarchy.h>
 
 #include "exchange-component.h"
+#include <e2k-utils.h>
 
 #ifndef O_BINARY
 #define O_BINARY 0
@@ -84,6 +85,9 @@ static GObjectClass *parent_class = NULL;
 
 #define d(x)
 
+static icaltimezone *
+internal_get_timezone (ECalBackend *backend, const char *tzid);
+
 static ECalBackendSyncStatus
 is_read_only (ECalBackendSync *backend, EDataCal *cal, gboolean *read_only)
 {
@@ -548,11 +552,50 @@ e_cal_backend_exchange_cache_sync_start (ECalBackendExchange *cbex)
 	g_hash_table_foreach (cbex->priv->objects, add_to_unseen, cbex);
 }
 
+static gboolean
+find_instance (ECalBackendExchange *cbex, ECalBackendExchangeComponent *ecomp, const char *rid, const char *lastmod)
+{
+	GList *l;
+	gboolean found = FALSE;
+
+	if (!ecomp->instances)	
+		return FALSE;
+
+	for (l = ecomp->instances; l != NULL; l = l->next) {
+		ECalComponent *comp = e_cal_component_new ();
+		ECalComponentRange recur_id;
+		struct icaltimetype inst_rid, new_rid;
+		time_t rtime;
+		icaltimezone *f_zone;
+
+		e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (l->data));
+		e_cal_component_get_recurid (comp, &recur_id);
+
+		rtime = e2k_parse_timestamp (rid);
+		new_rid = icaltime_from_timet (rtime, FALSE);
+
+		f_zone = internal_get_timezone ((ECalBackend *) cbex, recur_id.datetime.tzid);
+		recur_id.datetime.value->zone = f_zone;
+		inst_rid = icaltime_convert_to_zone (*recur_id.datetime.value, icaltimezone_get_utc_timezone ());
+
+		e_cal_component_free_datetime (&recur_id.datetime);
+		g_object_unref (comp);
+		
+		if (icaltime_compare (inst_rid, new_rid) == 0) {
+			found = TRUE;
+			break;
+		}
+	}
+	return found;
+}
+
 gboolean
 e_cal_backend_exchange_in_cache (ECalBackendExchange *cbex,
 				 const char          *uid,
 				 const char          *lastmod,
-				 const char	     *href)
+				 const char	     *href,
+				 const char	     *rid
+				 )
 {
 	ECalBackendExchangeComponent *ecomp;
 
@@ -563,6 +606,9 @@ e_cal_backend_exchange_in_cache (ECalBackendExchange *cbex,
 		return FALSE;
 	g_hash_table_remove (cbex->priv->cache_unseen, ecomp->uid);
 
+	if (rid) 
+		return find_instance (cbex, ecomp, rid, lastmod);
+
 	if (strcmp (ecomp->lastmod, lastmod) < 0) {
 		g_hash_table_remove (cbex->priv->objects, uid);
 		return FALSE;
@@ -1830,7 +1876,6 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const char *subject,
 	char *from_name = NULL, *from_email = NULL;
 	GSList *attach_list = NULL, *l, *new_attach_list = NULL;
 	char *fname, *file_contents = NULL, *filename, *dest_url, *mime_filename, *attach_file;
-	char *mime_type;
 	int len = 0;
 
 	if (!g_ascii_strcasecmp(e_cal_backend_exchange_get_owner_email (E_CAL_BACKEND_SYNC (cbex)), exchange_account_get_email_id (cbex->account)))
@@ -2022,6 +2067,15 @@ internal_get_timezone (ECalBackend *backend, const char *tzid)
 	return g_hash_table_lookup (cbex->priv->timezones, tzid);
 }
 
+icaltimezone *
+e_cal_backend_exchange_lookup_timezone (const char *tzid,
+					const void *custom,
+					GError **error)
+{
+	return internal_get_timezone (E_CAL_BACKEND ((ECalBackendExchange *)custom),
+				      tzid);
+}
+
 
 static void
 free_exchange_comp (gpointer value)
